@@ -1,24 +1,21 @@
 import datetime
 import os
 import shutil
-import StringIO
 import stat
 import tempfile
 from textwrap import dedent
 
+import mock
+
 from testify import assert_equal, assert_in
 from testify import run, setup, teardown, TestCase
-from tron.config import config_parse, schema
-from tron.config.config_parse import TronConfig, load_config, ConfigSSHOptions, valid_identifier
-from tron.config.config_parse import valid_job, valid_output_stream_dir
-from tron.config.config_parse import ConfigNode, ConfigNodePool, ConfigJob
-from tron.config.config_parse import ConfigAction, ConfigCleanupAction
-from tron.config.config_parse import ConfigService, ConfigError
-from tron.config.config_parse import CLEANUP_ACTION_NAME
-from tron.config.config_parse import valid_node_pool, valid_config
+import yaml
+from tron.config import config_parse, schema, manager
+from tron.config.config_parse import *
 from tron.config.schedule_parse import ConfigConstantScheduler
 from tron.config.schedule_parse import ConfigGrocScheduler
 from tron.config.schedule_parse import ConfigIntervalScheduler
+from tron.config.schema import MASTER_NAMESPACE
 from tests.assertions import assert_raises
 from tron.utils.dicts import FrozenDict
 
@@ -41,6 +38,10 @@ node_pools:
 """
 
 
+def valid_config_from_yaml(config_content):
+    return valid_config(manager.from_string(config_content))
+
+
 class ConfigTestCase(TestCase):
     BASE_CONFIG = """
 output_stream_dir: "/tmp"
@@ -139,16 +140,7 @@ services:
         monitor_interval: 20
 """
 
-    @setup
-    def setup_testdir(self):
-        self.test_dir = tempfile.mkdtemp()
-
-    @teardown
-    def teardown_testdir(self):
-        shutil.rmtree(self.test_dir)
-
     def test_attributes(self):
-        test_config = load_config(StringIO.StringIO(self.config))
         expected = TronConfig(
             output_stream_dir='/tmp',
             command_context=FrozenDict({
@@ -163,8 +155,8 @@ services:
             time_zone=None,
             state_persistence=config_parse.DEFAULT_STATE_PERSISTENCE,
             nodes=FrozenDict({
-                'node0': ConfigNode(name='node0', hostname='node0'),
-                'node1': ConfigNode(name='node1', hostname='node1')
+                'node0': ConfigNode(name='node0', username=os.environ['USER'], hostname='node0'),
+                'node1': ConfigNode(name='node1', username=os.environ['USER'], hostname='node1')
             }),
             node_pools=FrozenDict({
                 'nodePool': ConfigNodePool(nodes=['node0', 'node1'],
@@ -307,8 +299,7 @@ services:
             )
         )
 
-        # we could just do a big assert_equal here, but it would be hella hard
-        # to debug failures that way.
+        test_config = valid_config_from_yaml(self.config)
         assert_equal(test_config.command_context, expected.command_context)
         assert_equal(test_config.ssh_options, expected.ssh_options)
         assert_equal(test_config.notification_options, expected.notification_options)
@@ -326,6 +317,232 @@ services:
         assert_equal(test_config.jobs['test_job4'].enabled, False)
 
 
+class NamedConfigTestCase(TestCase):
+    config = """
+jobs:
+    -
+        name: "test_job0"
+        node: node0
+        schedule: "interval 20s"
+        actions:
+            -
+                name: "action0_0"
+                command: "test_command0.0"
+        cleanup_action:
+            command: "test_command0.1"
+
+    -
+        name: "test_job1"
+        node: node0
+        schedule: "daily 00:30:00 MWF"
+        allow_overlap: True
+        actions:
+            -
+                name: "action1_0"
+                command: "test_command1.0"
+            -
+                name: "action1_1"
+                command: "test_command1.1"
+                requires: [action1_0]
+
+    -
+        name: "test_job2"
+        node: node1
+        schedule: "daily 16:30:00"
+        actions:
+            -
+                name: "action2_0"
+                command: "test_command2.0"
+
+    -
+        name: "test_job3"
+        node: node1
+        schedule: "constant"
+        actions:
+            -
+                name: "action3_0"
+                command: "test_command3.0"
+            -
+                name: "action3_1"
+                command: "test_command3.1"
+            -
+                name: "action3_2"
+                node: node0
+                command: "test_command3.2"
+                requires: [action3_0, action3_1]
+
+    -
+        name: "test_job4"
+        node: NodePool
+        all_nodes: True
+        schedule: "daily"
+        enabled: False
+        actions:
+            -
+                name: "action4_0"
+                command: "test_command4.0"
+
+services:
+    -
+        name: "service0"
+        node: NodePool
+        command: "service_command0"
+        count: 2
+        pid_file: "/var/run/%(name)s-%(instance_number)s.pid"
+        monitor_interval: 20
+"""
+
+    def test_attributes(self):
+        expected = NamedTronConfig(
+            jobs=FrozenDict({
+                'test_job0': ConfigJob(
+                    name='test_job0',
+                    node='node0',
+                    schedule=ConfigIntervalScheduler(
+                        timedelta=datetime.timedelta(0, 20)),
+                    actions=FrozenDict({
+                        'action0_0': ConfigAction(
+                            name='action0_0',
+                            command='test_command0.0',
+                            requires=(),
+                            node=None)
+                    }),
+                    queueing=True,
+                    run_limit=50,
+                    all_nodes=False,
+                    cleanup_action=ConfigCleanupAction(
+                        name='cleanup',
+                        command='test_command0.1',
+                        requires=(),
+                        node=None),
+                    enabled=True,
+                    allow_overlap=False),
+                'test_job1': ConfigJob(
+                    name='test_job1',
+                    node='node0',
+                    enabled=True,
+                    schedule=ConfigGrocScheduler(
+                        ordinals=None,
+                        weekdays=set([1, 3, 5]),
+                        monthdays=None,
+                        months=None,
+                        timestr='00:30',
+                    ),
+                    actions=FrozenDict({
+                        'action1_1': ConfigAction(
+                            name='action1_1',
+                            command='test_command1.1',
+                            requires=('action1_0',),
+                            node=None),
+                        'action1_0': ConfigAction(
+                            name='action1_0',
+                            command='test_command1.0',
+                            requires=(),
+                            node=None)
+                    }),
+                    queueing=True,
+                    run_limit=50,
+                    all_nodes=False,
+                    cleanup_action=None,
+                    allow_overlap=True),
+                'test_job2': ConfigJob(
+                    name='test_job2',
+                    node='node1',
+                    enabled=True,
+                    schedule=ConfigGrocScheduler(
+                        ordinals=None,
+                        weekdays=None,
+                        monthdays=None,
+                        months=None,
+                        timestr='16:30',
+                    ),
+                    actions=FrozenDict({
+                        'action2_0': ConfigAction(
+                            name='action2_0',
+                            command='test_command2.0',
+                            requires=(),
+                            node=None)
+                    }),
+                    queueing=True,
+                    run_limit=50,
+                    all_nodes=False,
+                    cleanup_action=None,
+                    allow_overlap=False),
+                'test_job3': ConfigJob(
+                    name='test_job3',
+                    node='node1',
+                    schedule=ConfigConstantScheduler(),
+                    enabled=True,
+                    actions=FrozenDict({
+                        'action3_1': ConfigAction(
+                            name='action3_1',
+                            command='test_command3.1',
+                            requires=(),
+                            node=None),
+                        'action3_0': ConfigAction(
+                            name='action3_0',
+                            command='test_command3.0',
+                            requires=(),
+                            node=None),
+                        'action3_2': ConfigAction(
+                            name='action3_2',
+                            command='test_command3.2',
+                            requires=('action3_0', 'action3_1'),
+                            node='node0')
+                    }),
+                    queueing=True,
+                    run_limit=50,
+                    all_nodes=False,
+                    cleanup_action=None,
+                    allow_overlap=False),
+                'test_job4': ConfigJob(
+                    name='test_job4',
+                    node='NodePool',
+                    schedule=ConfigGrocScheduler(
+                        ordinals=None,
+                        weekdays=None,
+                        monthdays=None,
+                        months=None,
+                        timestr='00:00',
+                    ),
+                    actions=FrozenDict({
+                        'action4_0': ConfigAction(
+                            name='action4_0',
+                            command='test_command4.0',
+                            requires=(),
+                            node=None)}),
+                    queueing=True,
+                    run_limit=50,
+                    all_nodes=True,
+                    cleanup_action=None,
+                    enabled=False,
+                    allow_overlap=False)
+                }),
+                services=FrozenDict({
+                    'service0': ConfigService(
+                        name='service0',
+                        node='NodePool',
+                        pid_file='/var/run/%(name)s-%(instance_number)s.pid',
+                        command='service_command0',
+                        monitor_interval=20,
+                        restart_interval=None,
+                        count=2)
+                }
+            )
+        )
+
+        test_config = valid_named_config(yaml.load(self.config))
+        assert_equal(test_config.jobs['test_job0'], expected.jobs['test_job0'])
+        assert_equal(test_config.jobs['test_job1'], expected.jobs['test_job1'])
+        assert_equal(test_config.jobs['test_job2'], expected.jobs['test_job2'])
+        assert_equal(test_config.jobs['test_job3'], expected.jobs['test_job3'])
+        assert_equal(test_config.jobs['test_job4'], expected.jobs['test_job4'])
+        assert_equal(test_config.jobs, expected.jobs)
+        assert_equal(test_config.services, expected.services)
+        assert_equal(test_config, expected)
+        assert_equal(test_config.jobs['test_job4'].enabled, False)
+
+
 class JobConfigTestCase(TestCase):
 
     def test_no_actions(self):
@@ -337,7 +554,7 @@ jobs:
         schedule: "interval 20s"
         """
         expected_message = "Job test_job0 is missing options: actions"
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expected_message, str(exception))
 
     def test_empty_actions(self):
@@ -350,7 +567,7 @@ jobs:
         actions:
         """
         expected_message = "Value at Job.test_job0 is not a list with items"
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expected_message, str(exception))
 
     def test_dupe_names(self):
@@ -370,7 +587,7 @@ jobs:
 
         """
         expected_message = "Action name action0_0 on job test_job0 used twice"
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expected_message, str(exception))
 
     def test_bad_requires(self):
@@ -401,7 +618,7 @@ jobs:
         """
         expected_message = ('jobs.test_job1.action1_0 has a dependency '
                 '"action0_0" that is not in the same job!')
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expected_message, str(exception))
 
 
@@ -423,7 +640,7 @@ jobs:
                 requires: action0_0
         """
         expect = "Circular dependency in job.test_job0: action0_0 -> action0_1"
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expect, exception)
 
     def test_config_cleanup_name_collision(self):
@@ -440,7 +657,7 @@ jobs:
 
         """ % CLEANUP_ACTION_NAME
         expected_message = "Bad action name at Action.cleanup: cleanup"
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expected_message, str(exception))
 
     def test_config_cleanup_action_name(self):
@@ -459,7 +676,7 @@ jobs:
             command: "test_command0.1"
         """
         expected_msg = "Cleanup actions cannot have custom names"
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expected_msg, str(exception))
 
     def test_config_cleanup_requires(self):
@@ -478,7 +695,7 @@ jobs:
             requires: [action0_0]
         """
         expected_msg = "can not have requires"
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expected_msg, str(exception))
 
     def test_job_in_services(self):
@@ -496,7 +713,7 @@ services:
             command: "test_command0.1"
 """
         expected_msg = "Service test_job0 is missing options:"
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expected_msg, str(exception))
 
     def test_overlap_job_service_names(self):
@@ -570,7 +787,7 @@ class NodeConfigTestCase(TestCase):
                             command: "test_command0.0"
             """)
         expected_msg = "some_unknown_node configured for ConfigJob test_job0"
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expected_msg, str(exception))
 
     def test_invalid_nested_node_pools(self):
@@ -593,7 +810,7 @@ class NodeConfigTestCase(TestCase):
                       command: "echo 1"
         """)
         expected_msg = "NodePool pool1 contains another NodePool pool0"
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expected_msg, str(exception))
 
     def test_invalid_node_pool_config(self):
@@ -616,10 +833,112 @@ class NodeConfigTestCase(TestCase):
                       command: "echo 1"
         """)
         expected_msg = "NodePool pool0 is missing options"
-        exception = assert_raises(ConfigError, load_config, test_config)
+        exception = assert_raises(ConfigError, valid_config_from_yaml, test_config)
         assert_in(expected_msg, str(exception))
 
 
+    def test_invalid_named_update(self):
+        test_config = """bozray:"""
+        test_config = yaml.load(test_config)
+        expected_message = "Unknown keys in NamedConfigFragment : bozray"
+        exception = assert_raises(ConfigError, validate_fragment, 'foo', test_config)
+        assert_in(expected_message, str(exception))
+
+
+class CollateJobsAndServicesTestCase(TestCase):
+
+    def test_valid_job_collation(self):
+        test_config = BASE_CONFIG + """
+jobs:
+    -
+        name: "test_job0"
+        node: node0
+        schedule: "interval 20s"
+        actions:
+            -
+                name: "action0_0"
+                command: "test_command0.0"
+        cleanup_action:
+            command: "test_command0.1"
+services:
+    -
+        name: "test_service0"
+        node: node0
+        command: "service_command0"
+        count: 2
+        pid_file: "/var/run/%(name)s-%(instance_number)s.pid"
+        monitor_interval: 20
+        """
+        expected_collated_jobs = {'MASTER.test_job0':
+                ConfigJob(name='MASTER.test_job0',
+                          node='node0',
+                          schedule=ConfigIntervalScheduler(timedelta=datetime.timedelta(0, 20)),
+                          actions=FrozenDict({'action0_0':
+                                              ConfigAction(name='action0_0',
+                                                           command='test_command0.0',
+                                                           requires=(),
+                                                           node=None)}),
+                          queueing=True,
+                          run_limit=50,
+                          all_nodes=False,
+                          cleanup_action=ConfigCleanupAction(command='test_command0.1',
+                                                             requires=(),
+                                                             name='cleanup',
+                                                             node=None),
+                          enabled=True,
+                          allow_overlap=False),
+                }
+
+        expected_collated_services = {'MASTER.test_service0':
+                ConfigService(name='MASTER.test_service0',
+                              node='node0',
+                              pid_file='/var/run/%(name)s-%(instance_number)s.pid',
+                              command='service_command0',
+                              monitor_interval=20,
+                              restart_interval=None,
+                              count=2),
+                }
+
+        config_mapping = {MASTER_NAMESPACE: valid_config_from_yaml(test_config)}
+        config_container = ConfigContainer(config_mapping)
+        jobs, services = collate_jobs_and_services(config_container)
+        assert_equal(expected_collated_jobs, jobs)
+        assert_equal(expected_collated_services, services)
+
+    def test_invalid_job_collation(self):
+        jobs = FrozenDict({'test_collision0': ConfigJob(name='test_collision0',
+            node='node0',
+            schedule=ConfigIntervalScheduler(timedelta=datetime.timedelta(0,
+                                                                          20)),
+            actions=FrozenDict({'action0_0': ConfigAction(name='action0_0',
+                                                          command='test_command0.0',
+                                                          requires=(),
+                                                          node=None)}),
+            queueing=True,
+            run_limit=50,
+            all_nodes=False,
+            cleanup_action=ConfigCleanupAction(command='test_command0.1',
+                                               requires=(),
+                                               name='cleanup',
+                                               node=None),
+            enabled=True,
+            allow_overlap=False)})
+
+        services = FrozenDict({'test_collision0': ConfigService(name='test_collision0',
+                        node='node0',
+                        pid_file='/var/run/%(name)s-%(instance_number)s.pid',
+                        command='service_command0',
+                        monitor_interval=20,
+                        restart_interval=None,
+                        count=2)})
+        fake_config = mock.Mock()
+        setattr(fake_config, 'jobs', jobs)
+        setattr(fake_config, 'services', services)
+        expected_message = "Collision found for identifier 'MASTER.test_collision0'"
+        exception = assert_raises(ConfigError, collate_jobs_and_services, {'MASTER': fake_config})
+        assert_in(expected_message, str(exception))
+
+
 StubConfigObject = schema.config_object_factory(
     'StubConfigObject',
     ['req1', 'req2'],
@@ -683,5 +1002,67 @@ class ValidatorIdentifierTestCase(TestCase):
             assert_raises(ConfigError, valid_identifier, '', name)
 
 
+class ConfigContainerTestCase(TestCase):
+
+    @setup
+    def setup_container(self):
+        other_config = yaml.load(NamedConfigTestCase.config)
+        self.config_mapping = {
+            MASTER_NAMESPACE: valid_config(yaml.load(BASE_CONFIG)),
+            'other': validate_fragment('other', other_config)}
+        self.container = config_parse.ConfigContainer(self.config_mapping)
+
+    @mock.patch('tron.config.config_parse.ConfigContainer.validate', autospec=True)
+    @mock.patch('tron.config.config_parse.validate_fragment', autospec=True)
+    def test_create(self, mock_validate_fragment, mock_validate):
+        config_mapping = {
+            MASTER_NAMESPACE: yaml.load(BASE_CONFIG),
+            'other': yaml.load(NamedConfigTestCase.config)}
+
+        def validate_frag(name, config):
+            return name, config
+        mock_validate_fragment.side_effect = validate_frag
+        container = config_parse.ConfigContainer.create(config_mapping)
+        expected = [mock.call(k, v) for k, v in config_mapping.iteritems()]
+        assert_equal(mock_validate_fragment.mock_calls, expected)
+        mock_validate.assert_called_with(container)
+        assert container.configs
+
+    def test_create_missing_master(self):
+        config_mapping = {'other': mock.Mock()}
+        assert_raises(ConfigError,
+            config_parse.ConfigContainer.create, config_mapping)
+
+    @mock.patch('tron.config.config_parse.collate_jobs_and_services', autospec=True)
+    def test_validate(self, mock_collate):
+        self.container.validate()
+        mock_collate.assert_called_with(self.container)
+
+    def test_validate_unknown_nodes(self):
+        service = {
+            'name': 'some_name',
+            'node': 'unknown_node',
+            'command': 'do things',
+            'pid_file': '/tmp',
+            'monitor_interval': 30
+        }
+        config = {'services': [service]}
+        self.container.add('third', config)
+        assert_raises(ConfigError, self.container.validate)
+
+    @mock.patch('tron.config.config_parse.validate_fragment', autospec=True)
+    def test_add(self, mock_validate_fragment):
+        name, content = 'name', mock.Mock()
+        self.container.add(name, content)
+        assert_equal(self.container.configs[name],
+            mock_validate_fragment.return_value)
+        mock_validate_fragment.assert_called_with(name, content)
+
+    def test_get_node_names(self):
+        node_names = self.container.get_node_names()
+        expected = set(['node0', 'node1', 'NodePool'])
+        assert_equal(node_names, expected)
+
+
 if __name__ == '__main__':
     run()
