@@ -19,26 +19,51 @@ import os
 import re
 
 import pytz
-import yaml
 
 from tron.config import ConfigError
 from tron.config.schedule_parse import valid_schedule
-from tron.config.schema import TronConfig, NotificationOptions, ConfigSSHOptions
+from tron.config.schema import TronConfig, NamedTronConfig, NotificationOptions, ConfigSSHOptions
 from tron.config.schema import ConfigNode, ConfigNodePool, ConfigState
 from tron.config.schema import ConfigJob, ConfigAction, ConfigCleanupAction
 from tron.config.schema import ConfigService
+from tron.config.schema import MASTER_NAMESPACE
 from tron.utils.dicts import FrozenDict
 from tron.core.action import CLEANUP_ACTION_NAME
 
 
 log = logging.getLogger(__name__)
 
-def load_config(config):
-    """Given a string or file object, load it with PyYAML and return an
-    immutable, validated representation of the configuration it specifies.
-    """
-    # Load with YAML. safe_load() disables python classes
-    return valid_config(yaml.safe_load(config))
+
+# TODO(0.6): move to ConfigContainer
+def collate_jobs_and_services(configs):
+    """Collate jobs and services from an iterable of Config objects."""
+    jobs = {}
+    services = {}
+
+    def build_identifier(namespace, name):
+        return '%s.%s' % (namespace, name)
+
+    # TODO(0.6) remove once inline
+    def repack_with_identifier(identifier, item):
+        return type(item)(identifier, *item[1:])
+
+    def _iter_items(config, namespace, attr):
+        for item in getattr(config, attr):
+
+            identifier = build_identifier(namespace, item)
+            if identifier in jobs or identifier in services:
+                raise ConfigError("Collision found for identifier '%s'" % job_identifier)
+            content = getattr(config, attr)[item]
+            yield identifier, repack_with_identifier(identifier, content)
+
+    for namespace, config in configs.iteritems():
+        for job_identifier, content in _iter_items(config, namespace, "jobs"):
+            jobs[job_identifier] = content
+
+        for service_identifier, content in _iter_items(config, namespace, "services"):
+            services[service_identifier] = content
+
+    return jobs, services
 
 
 class UniqueNameDict(dict):
@@ -187,7 +212,7 @@ class Validator(object):
         if not extra_keys:
             return
 
-        msg             = "Unknown options in %s %s: %s"
+        msg             = "Unknown keys in %s %s: %s"
         name            = in_dict.get('name', '')
         raise ConfigError(msg % (self.type_name, name, ', '.join(extra_keys)))
 
@@ -294,6 +319,7 @@ class ValidateNode(Validator):
     config_class =              ConfigNode
     validators = {
         'name':                 partial(valid_identifier, 'nodes'),
+        'username':             partial(valid_str, 'nodes'),
         'hostname':             partial(valid_str, 'nodes')
     }
 
@@ -304,6 +330,7 @@ class ValidateNode(Validator):
 
     def set_defaults(self, output_dict):
         output_dict.setdefault('name', output_dict['hostname'])
+        output_dict.setdefault('username', os.environ['USER'])
 
 valid_node = ValidateNode()
 
@@ -523,6 +550,24 @@ valid_state_persistence = ValidateStatePersistence()
 
 DEFAULT_STATE_PERSISTENCE = ConfigState('tron_state', 'shelve', None, 1)
 
+def parse_sub_config(config, cname, valid, name_dict):
+    target_dict = UniqueNameDict(
+        "%s name %%s used twice" % cname.replace('_', ' '))
+    for item in config.get(cname) or []:
+        final = valid(item)
+        target_dict[final.name] = final
+        name_dict[final.name] = True
+    config[cname] = FrozenDict(**target_dict)
+
+def validate_jobs_and_services(config):
+    """Validate jobs and services."""
+
+    job_service_names = UniqueNameDict(
+            'Job and Service names must be unique %s')
+
+    parse_sub_config(config, 'jobs',        valid_job ,         job_service_names)
+    parse_sub_config(config, 'services',    valid_service,      job_service_names)
+
 
 class ValidateConfig(Validator):
     """Given a parsed config file (should be only basic literals and
@@ -538,7 +583,7 @@ class ValidateConfig(Validator):
         'notification_options': None,
         'time_zone':            None,
         'state_persistence':    DEFAULT_STATE_PERSISTENCE,
-        'nodes':                (dict(name='localhost', hostname='localhost'),),
+        'nodes':                (dict(name='localhost', username='tronuser', hostname='localhost'),),
         'node_pools':           (),
         'jobs':                 (),
         'services':             (),
@@ -553,47 +598,6 @@ class ValidateConfig(Validator):
     }
     optional = False
 
-    def post_validation(self, config):
-        """Validate jobs, nodes, and services."""
-
-        node_names = UniqueNameDict('Node and NodePool names must be unique %s')
-        job_service_names = UniqueNameDict(
-                'Job and Service names must be unique %s')
-
-        def parse_sub_config(cname, valid, name_dict):
-            target_dict = UniqueNameDict(
-                "%s name %%s used twice" % cname.replace('_', ' '))
-            for item in config.get(cname) or []:
-                final = valid(item)
-                target_dict[final.name] = final
-                name_dict[final.name] = True
-            config[cname] = FrozenDict(**target_dict)
-
-        parse_sub_config('nodes',       valid_node,         node_names)
-        parse_sub_config('node_pools',  valid_node_pool,    node_names)
-        parse_sub_config('jobs',        valid_job ,         job_service_names)
-        parse_sub_config('services',    valid_service,      job_service_names)
-
-        self.validate_node_names(config, node_names)
-        self.validate_node_pool_nodes(config)
-
-    def validate_node_names(self, config, node_names):
-        """Validate that any node/node_pool name that were used are configured
-        as nodes/node_pools.
-        """
-        actions = itertools.chain.from_iterable(
-            job.actions.values()
-            for job in config['jobs'].values())
-
-        task_list = itertools.chain(
-            config['jobs'].values(),
-            config['services'].values(),
-            actions)
-        for task in task_list:
-            if task.node and task.node not in node_names:
-                raise ConfigError("Unknown node %s configured for %s %s" % (
-                    task.node, task.__class__.__name__, task.name))
-
     def validate_node_pool_nodes(self, config):
         """Validate that each node in a node_pool is in fact a node, and not
         another pool.
@@ -606,4 +610,110 @@ class ValidateConfig(Validator):
                 msg = "NodePool %s contains another NodePool %s. "
                 raise ConfigError(msg % (node_pool.name, node_name))
 
+    def post_validation(self, config):
+        """Validate a non-named config."""
+        validate_jobs_and_services(config)
+        node_names = UniqueNameDict('Node and NodePool names must be unique %s')
+        parse_sub_config(config, 'nodes',       valid_node,         node_names)
+        parse_sub_config(config, 'node_pools',  valid_node_pool,    node_names)
+        validate_node_names(config['jobs'], config['services'], node_names)
+        self.validate_node_pool_nodes(config)
+
+
+# TODO(0.6): validate inline in Validation class
+def validate_node_names(jobs, services, node_names):
+
+    def tasks_from_job(job):
+        for action in job.actions.itervalues():
+            yield action
+        if job.cleanup_action:
+            yield job.cleanup_action
+        yield job
+
+    job_tasks = itertools.chain.from_iterable(
+                    tasks_from_job(job) for job in jobs.itervalues())
+    task_list = itertools.chain(job_tasks, services.itervalues())
+    for task in task_list:
+        if task.node and task.node not in node_names:
+            raise ConfigError("Unknown node %s configured for %s %s" % (
+                task.node, task.__class__.__name__, task.name))
+
+
+class ValidateNamedConfig(Validator):
+    """A shorter validator for named configurations, which allow for
+    jobs and services to be defined as configuration fragments that
+    are, in turn, reconciled by Tron.
+    """
+    config_class =              NamedTronConfig
+    type_name =                 "NamedConfigFragment"
+    defaults = {
+        'jobs':                 (),
+        'services':             ()
+    }
+    optional = False
+
+    def post_validation(self, config):
+        """Validate a named config."""
+        validate_jobs_and_services(config)
+
 valid_config = ValidateConfig()
+valid_named_config = ValidateNamedConfig()
+
+
+def validate_fragment(name, fragment):
+    if name == MASTER_NAMESPACE:
+        return valid_config(fragment)
+    return valid_named_config(fragment)
+
+
+class ConfigContainer(object):
+    """A container around configuration fragments (and master)."""
+
+    def __init__(self, config_mapping):
+        self.configs = config_mapping
+
+    def iteritems(self):
+        return self.configs.iteritems()
+
+    @classmethod
+    def create(cls, config_mapping):
+        if MASTER_NAMESPACE not in config_mapping:
+            msg = "%s requires a %s"
+            raise ConfigError(msg % (cls.__name__, MASTER_NAMESPACE))
+
+        def build_mapping():
+            for name, content in config_mapping.iteritems():
+                yield name, validate_fragment(name, content)
+
+        container = cls(dict(build_mapping()))
+        container.validate()
+        return container
+
+    def validate(self):
+        """Validate the integrity of all the configuration fragments as a whole.
+        """
+        collate_jobs_and_services(self)
+        node_names = self.get_node_names()
+        for name, fragment in self.iteritems():
+            validate_node_names(fragment.jobs, fragment.services, node_names)
+
+    # TODO(0.6) remove once names are compiled inline
+    def get_job_and_service_names(self):
+        jobs, services = collate_jobs_and_services(self)
+        return jobs.keys(), services.keys()
+
+    def add(self, name, config_content):
+        self.configs[name] = validate_fragment(name, config_content)
+
+    def get_master(self):
+        return self.configs[MASTER_NAMESPACE]
+
+    def get_node_names(self):
+        master = self.get_master()
+        return set(itertools.chain(master.nodes, master.node_pools))
+
+    def __getitem__(self, name):
+        return self.configs[name]
+
+    def __contains__(self, name):
+        return name in self.configs
