@@ -3,6 +3,7 @@
 """
 from collections import namedtuple
 
+MASTER_NAMESPACE = "MASTER"
 
 def config_object_factory(name, required=None, optional=None):
     """
@@ -37,6 +38,13 @@ TronConfig = config_object_factory(
         'services'             # FrozenDict of ConfigService
     ])
 
+NamedTronConfig = config_object_factory(
+    'NamedTronConfig',
+    optional=[
+        'jobs',                # FrozenDict of ConfigJob
+        'services'             # FrozenDict of ConfigService
+    ])
+
 
 NotificationOptions = config_object_factory(
     'NotificationOptions',
@@ -54,7 +62,7 @@ ConfigSSHOptions = config_object_factory(
     ])
 
 
-ConfigNode = config_object_factory('ConfigNode', ['hostname'], ['name'])
+ConfigNode = config_object_factory('ConfigNode', ['hostname'], ['name', 'username'])
 
 
 ConfigNodePool = config_object_factory('ConfigNodePool', ['nodes'], ['name'])
@@ -120,4 +128,4 @@ ConfigService = config_object_factory(
     ],[
         'restart_interval',     # float
         'count',                # int
-    ])
\ No newline at end of file
+    ])
