@@ -8,13 +8,13 @@ from tron import command_context
 from tron import event
 from tron import crash_reporter
 from tron import node
-from tron.config import config_parse
-from tron.config.config_parse import ConfigError
+from tron.config import manager
+from tron.config.config_parse import collate_jobs_and_services, ConfigError
 from tron.core.job import Job, JobScheduler
 from tron.node import Node, NodePool
 from tron.scheduler import scheduler_from_config
 from tron.serialize import filehandler
-from tron.serialize.runstate.statemanager import PersistenceManagerFactory
+from tron.serialize.runstate import statemanager
 from tron.service import Service
 from tron.utils import emailer
 from tron.utils.observer import Observable
@@ -34,7 +34,7 @@ class MasterControlProgram(Observable):
     Central state object for the Tron daemon. Stores all jobs and services.
     """
 
-    def __init__(self, working_dir, config_file):
+    def __init__(self, working_dir, config_path):
         super(MasterControlProgram, self).__init__()
         self.jobs               = {}
         self.services           = {}
@@ -42,8 +42,9 @@ class MasterControlProgram(Observable):
         self.output_stream_dir  = None
         self.working_dir        = working_dir
         self.crash_reporter     = None
-        self.config_filepath    = config_file
+        self.config             = manager.ConfigManager(config_path)
         self.context            = command_context.CommandContext()
+        self.state_watcher      = statemanager.StateChangeWatcher()
 
         # Time zone of the system clock
         self.time_zone          = None
@@ -54,12 +55,12 @@ class MasterControlProgram(Observable):
         # tree.
         self.event_manager      = event.EventManager.get_instance()
         self.event_recorder     = self.event_manager.add(self)
-        self.state_manager      = None
+
+    def get_config_manager(self):
+        return self.config
 
     def shutdown(self):
-        if self.state_manager:
-            self.state_manager.enabled = False
-            self.state_manager.cleanup()
+        self.state_watcher.shutdown()
 
     def graceful_shutdown(self):
         """Tell JobSchedulers that a shutdown has been requested."""
@@ -75,7 +76,7 @@ class MasterControlProgram(Observable):
     def reconfigure(self):
         """Reconfigure MCP while Tron is already running."""
         self.event_recorder.emit_info("reconfig")
-        with self.state_manager.disabled():
+        with self.state_watcher.disabled():
             try:
                 self._load_config(reconfigure=True)
             except Exception:
@@ -85,10 +86,7 @@ class MasterControlProgram(Observable):
 
     def _load_config(self, reconfigure=False):
         """Read config data and apply it."""
-        log.info("Loading configuration from %s" % self.config_filepath)
-        with open(self.config_filepath, 'r') as f:
-            config = config_parse.load_config(f)
-        self.apply_config(config, reconfigure=reconfigure)
+        self.apply_config(self.config.load(), reconfigure=reconfigure)
 
     def initial_setup(self):
         """When the MCP is initialized the config is applied before the state.
@@ -100,32 +98,32 @@ class MasterControlProgram(Observable):
         # without any state will be scheduled here.
         self.schedule_jobs()
 
-    def apply_config(self, conf, skip_env_dependent=False, reconfigure=False):
-        """Apply a configuration. If skip_env_dependent is True we're
-        loading this locally to test the config as part of tronfig. We want to
-        skip applying some settings because the local machine we're using to
-        edit the config may not have the same environment as the live
-        trond machine.
+    def apply_config(self, config_container, reconfigure=False):
+        """Apply a configuration."""
+        master_config = config_container.get_master()
+        self.output_stream_dir = master_config.output_stream_dir or self.working_dir
+        ssh_options = self._ssh_options_from_config(master_config.ssh_options)
+        self.update_state_watcher_config(master_config.state_persistence)
+
+        self.context.base = master_config.command_context
+        self.time_zone = master_config.time_zone
+        self._apply_nodes(master_config.nodes, ssh_options)
+        self._apply_node_pools(master_config.node_pools)
+        self._apply_notification_options(master_config.notification_options)
+
+        jobs, services = collate_jobs_and_services(config_container)
+        self._apply_jobs(jobs, reconfigure=reconfigure)
+        self._apply_services(services)
+
+    def update_state_watcher_config(self, state_config):
+        """Update the StateChangeWatcher, and save all state if the state config
+        changed.
         """
-        self.output_stream_dir = conf.output_stream_dir or self.working_dir
-
-        if not skip_env_dependent:
-            ssh_options = self._ssh_options_from_config(conf.ssh_options)
-            state_persistence = conf.state_persistence
-        else:
-            ssh_options = config_parse.valid_ssh_options({})
-            state_persistence = config_parse.DEFAULT_STATE_PERSISTENCE
-
-        self.state_manager = PersistenceManagerFactory.from_config(
-                state_persistence)
-        self.context.base = conf.command_context
-        self.time_zone = conf.time_zone
-
-        self._apply_nodes(conf.nodes, ssh_options)
-        self._apply_node_pools(conf.node_pools)
-        self._apply_jobs(conf.jobs, reconfigure=reconfigure)
-        self._apply_services(conf.services)
-        self._apply_notification_options(conf.notification_options)
+        if self.state_watcher.update_from_config(state_config):
+            for job_sched in self.jobs.itervalues():
+                self.state_watcher.save_job(job_sched.job)
+            for service in self.services.itervalues():
+                self.state_watcher.save_service(service)
 
     def _ssh_options_from_config(self, ssh_conf):
         ssh_options = ConchOptions()
@@ -165,7 +163,7 @@ class MasterControlProgram(Observable):
 
     def _apply_jobs(self, job_configs, reconfigure=False):
         """Add and remove jobs based on the configuration."""
-        for job_config in job_configs.values():
+        for job_config in job_configs.itervalues():
             self.add_job(job_config, reconfigure=reconfigure)
 
         for job_name in (set(self.jobs.keys()) - set(job_configs.keys())):
@@ -176,7 +174,7 @@ class MasterControlProgram(Observable):
         """Add and remove services."""
 
         services_to_add = []
-        for srv_config in srv_configs.values():
+        for srv_config in srv_configs.itervalues():
             log.debug("Building new services %s", srv_config.name)
             service = Service.from_config(srv_config, self.nodes)
             services_to_add.append(service)
@@ -232,7 +230,7 @@ class MasterControlProgram(Observable):
         log.info("adding new job %s", job.name)
         self.jobs[job.name] = JobScheduler(job)
         self.event_manager.add(job, parent=self)
-        self.state_manager.watch(job, Job.NOTIFY_STATE_CHANGE)
+        self.state_watcher.watch(job, Job.NOTIFY_STATE_CHANGE)
 
         # If this is not a reconfigure, wait for state to be restored before
         # scheduling job runs.
@@ -257,6 +255,7 @@ class MasterControlProgram(Observable):
         return self.jobs.get(name)
 
     def add_service(self, service):
+
         if service.name in self.jobs:
             raise ValueError("Service %s is already a job", service.name)
 
@@ -270,7 +269,7 @@ class MasterControlProgram(Observable):
         service.event_recorder.set_parent(self.event_recorder)
 
         # Trigger storage on any state changes
-        self.state_manager.watch(service.machine)
+        self.state_watcher.watch(service.machine)
         self.services[service.name] = service
 
         if prev_service is not None:
@@ -289,9 +288,10 @@ class MasterControlProgram(Observable):
         to the configured Jobs and Services.
         """
         self.event_recorder.emit_notice('restoring')
-        job_states, service_states = self.state_manager.restore(
-                [job_sched.job for job_sched in self.jobs.values()],
-                self.services.values())
+        job_names     = [job_sched.job.name for job_sched in self.jobs.values()]
+        service_names = [service.name for service in self.services.itervalues()]
+        job_states, service_states = self.state_watcher.restore(
+                job_names, service_names)
 
         for name, job_state_data in job_states.iteritems():
             self.jobs[name].restore_job_state(job_state_data)
@@ -301,7 +301,7 @@ class MasterControlProgram(Observable):
             self.services[name].restore_service_state(service_state_data)
         log.info("Loaded state for %d services", len(service_states))
 
-        self.state_manager.save_metadata()
+        self.state_watcher.save_metadata()
 
     def __str__(self):
-        return "MCP"
+        return "MCP"
\ No newline at end of file
