@@ -1,5 +1,6 @@
 import logging
 import itertools
+import os
 import random
 
 from twisted.internet import protocol, defer, reactor
@@ -133,16 +134,18 @@ class Node(object):
     directly as a NodePool of 1 Node.
     """
 
-    def __init__(self, hostname=None, name=None, ssh_options=None):
+    def __init__(self, hostname, ssh_options, username=None, name=None):
+
         # Host we are to connect to
         self.hostname = hostname
 
+        # Username to connect to the remote machine under
+        self.username = username
+
         # Identifier for UI
         self.name = name or hostname
 
-        if not ssh_options or not hostname:
-            raise ValueError('Must specify hostname and ssh_options')
-
+        # SSH Options
         self.conch_options = ssh_options
 
         # The SSH connection we use to open channels on. If present, means we
@@ -161,9 +164,10 @@ class Node(object):
     @classmethod
     def from_config(cls, node_config, ssh_options):
         return cls(
-            hostname=node_config.hostname,
-            name=node_config.name,
-            ssh_options=ssh_options
+            node_config.hostname,
+            ssh_options,
+            username=node_config.username,
+            name=node_config.name
         )
 
     def next(self):
@@ -361,6 +365,7 @@ class Node(object):
         #  3. The connection service is started, so we can use it
 
         client_creator = protocol.ClientCreator(reactor, ssh.ClientTransport,
+                                                username=self.username,
                                                 options=self.conch_options)
         create_defer = client_creator.connectTCP(self.hostname, 22)
 
@@ -486,8 +491,12 @@ class Node(object):
         """Returns a dict which is an external view of this object."""
         return {
             'name':             self.name,
-            'hostname':         self.hostname
+            'hostname':         self.hostname,
+            'username':         self.username
         }
 
     def __str__(self):
-        return "Node:%s" % self.hostname
\ No newline at end of file
+        return "Node:%s@%s" % (self.username or "<default>", self.hostname)
+
+    def __repr__(self):
+        return self.__str__()
