@@ -1,5 +1,3 @@
-import os
-import pwd
 import struct
 import logging
 
@@ -23,23 +21,12 @@ class ChannelClosedEarlyError(Error):
     pass
 
 
-# We need to sub-class and redefine the AuthClient here because there is no way
-# with the default client to force it to not try certain authentication
-# methods. If things get much worse I'll just have to make our own custom
-# version of the default module.
 class NoPasswordAuthClient(default.SSHUserAuthClient):
-    def tryAuth(self, kind):
-        kind = kind.replace('-', '_')
-        if kind != 'publickey':
-            log.info('skipping auth method %s (not supported)' % kind)
-            return
+    """Only support passwordless auth."""
+    preferredOrder              = ['publickey']
+    auth_password               = None
+    auth_keyboard_interactive   = None
 
-        log.info('trying to auth with %s!' % kind)
-        f = getattr(self, 'auth_%s' % kind, None)
-        if f:
-            return f()
-        else:
-            return
 
 
 class ClientTransport(transport.SSHClientTransport):
@@ -51,20 +38,19 @@ class ClientTransport(transport.SSHClientTransport):
         # they're all old style classes
         # transport.SSHClientTransport.__init__(self, *args, **kwargs)
 
+        self.username = kwargs['username']
         self.options = kwargs['options']
 
     def verifyHostKey(self, pubKey, fingerprint):
+        # TODO: this should verify
         return defer.succeed(1)
 
     def connectionSecure(self):
         conn = ClientConnection()
         conn.service_defer = defer.Deferred()
-
         self.connection_defer.callback(conn)
 
-        auth_service = NoPasswordAuthClient(pwd.getpwuid(os.getuid())[0],
-                                            self.options, conn)
-
+        auth_service = NoPasswordAuthClient(self.username, self.options, conn)
         self.requestService(auth_service)
 
 
@@ -95,6 +81,24 @@ class ClientConnection(connection.SSHConnection):
 
         connection.SSHConnection.channelClosed(self, channel)
 
+    def ssh_CHANNEL_REQUEST(self, packet):
+        """
+        The other side is sending a request to a channel.  Payload::
+            uint32  local channel number
+            string  request name
+            bool    want reply
+            <request specific data>
+
+        Handles missing local channel.
+        """
+        localChannel = struct.unpack('>L', packet[: 4])[0]
+        if localChannel not in self.channels:
+            requestType, _ = common.getNS(packet[4:])
+            host = self.transport.transport.getHost()
+            msg = "Missing channel: %s, request_type: %s, host: %s"
+            log.warn(msg, localChannel, requestType, host)
+            return
+        connection.SSHConnection.ssh_CHANNEL_REQUEST(self, packet)
 
 class ExecChannel(channel.SSHChannel):
 
